use std::sync::{Arc, LazyLock, RwLock};

use bevy::{
    diagnostic::{DiagnosticsStore, FrameTimeDiagnosticsPlugin},
    prelude::*,
};

use crate::player::Player;

const FISH_COUNTER_OFFSET: Vec3 = Vec3::new(50.0, 50.0, 1.0);
static FISH_COUNT: LazyLock<Arc<RwLock<usize>>> = LazyLock::new(|| Arc::new(RwLock::new(0)));

#[derive(Debug, Component)]
pub struct FPSText;

impl FPSText {
    pub fn setup(mut commands: Commands) {
        commands.spawn((
            Text::new("FPS Uninitialized"),
            TextLayout::new_with_justify(JustifyText::Center),
            Node {
                position_type: PositionType::Absolute,
                ..default()
            },
            FPSText,
        ));
    }

    pub fn update(
        diagnostics: Res<DiagnosticsStore>,
        mut fps_text_query: Query<&mut Text, With<FPSText>>,
    ) {
        if let Some(fps) = diagnostics.get(&FrameTimeDiagnosticsPlugin::FPS) {
            if let Some(fps) = fps.smoothed() {
                if let Ok(mut fps_text) = fps_text_query.get_single_mut() {
                    *fps_text = format!("FPS = {:.2}", fps).into();
                }
            }
        }
    }
}

#[derive(Debug, Component)]
pub struct FishCounterText;

impl FishCounterText {
    pub fn setup(mut commands: Commands) {
        commands.spawn((
            Text2d::new("Fish Count = 0"),
            TextLayout::new_with_justify(JustifyText::Center),
            Transform::from_xyz(0.0, 0.0, 1.0),
            FishCounterText,
        ));
    }

    pub fn update(
        mut fish_counter_text_transform_query: Query<
            &mut Transform,
            (With<FishCounterText>, Without<Player>),
        >,
        mut fish_counter_text_query: Query<&mut Text2d, With<FishCounterText>>,
        player_transform_query: Query<&Transform, (With<Player>, Without<FishCounterText>)>,
        player_query: Query<&Player, With<Player>>,
    ) {
        let mut fish_counter_text_transform =
            fish_counter_text_transform_query.get_single_mut().unwrap();
        let mut fish_counter_text = fish_counter_text_query.get_single_mut().unwrap();
        let player = player_query.get_single().unwrap();

        let player_position = player_transform_query.get_single().unwrap();
        let mut new_fish_counter_text_position = player_position.translation.truncate().extend(0.0);
        new_fish_counter_text_position += FISH_COUNTER_OFFSET;
        fish_counter_text_transform.translation = new_fish_counter_text_position;
        if *FISH_COUNT.read().unwrap() != player.fish_count {
            *FISH_COUNT.write().unwrap() = player.fish_count;
            *fish_counter_text = format!("Fish Count = {}", player.fish_count).into();
        }
    }
}

#[derive(Debug, Component)]
pub struct KeybindingsText;

impl KeybindingsText {
    pub fn setup(mut commands: Commands) {
        commands.spawn((
            Text2d::new("Press E When You Are Near To Lake"),
            TextLayout::new_with_justify(JustifyText::Center),
            Transform::from_xyz(0.0, -300.0, 1.0),
        ));
    }
}
